/*
 Copyright (c) 2013 aizo ag, Zurich, Switzerland

 Author: Christain Hitz <christian.hitz@aizo.com>

 This file is part of dS Assistant.

 dS-Assistant is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 dS-Assistant is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.

 NOTE: all APIs, especially the JSON API, offered by this application
 are subject to change without further notice.
 */

#ifndef COMM_CHANNEL_H_
#define COMM_CHANNEL_H_

#include <libcommchannel/commchannel.h>
#include <sys/types.h>
#include <list>
#include <vector>

namespace dSA {

class CommChannel : public CC::CommunicationChannelCallback
{
public:
    static CommChannel* createInstance();
    static CommChannel* getInstance();
    virtual ~CommChannel();

    virtual void messageReceived(boost::shared_ptr<CC::CommunicationData> comm);

    void run();

    static std::string sendMessage(const std::string& message);
    static bool sendSceneLocks(const std::vector<int>& scenes);

private:
    CommChannel();
    void lockMessageList();
    void unlockMessageList();
    void shutdown();

    void taskThread();
    static void *staticThreadProc(void *arg);

private:
    static CommChannel* m_instance;
    int m_shutdown_flag;
    bool m_running;
    pthread_t m_thread;
    pthread_mutex_t m_mutex;
    pthread_cond_t m_condition;
    std::list<boost::shared_ptr<CC::CommunicationData> > m_datalist;
    CC::CommunicationChannelServer *m_pServer;

};

}

#endif /* COMM_CHANNEL_H_ */
